/**
 * Grader Extension API Schemas
 *
 * 
 *
 * NOTE: This class is auto generated by OpenAPI Generator (https://openapi-generator.tech).
 * https://openapi-generator.tech
 * Do not edit the class manually.
 */
import { FeedbackStatus } from './feedbackStatus';
import { ManualStatus } from './manualStatus';
import { AutoStatus } from './autoStatus';


export interface Submission { 
    id?: number;
    submitted_at?: string;
    auto_status?: AutoStatus;
    manual_status?: ManualStatus;
    user_id?: number;
    user_display_name?: string;
    grading_score?: number;
    score_scaling?: number;
    score?: number;
    assignid?: number;
    commit_hash?: string;
    feedback_status?: FeedbackStatus;
    edited?: boolean;
}
export namespace Submission {
}


