.grader-toolbar-button {
  text-transform: none !important;
  font-size: 9px !important;
  padding: 2px 3px !important;
  font-family: var(--jp-ui-font-family) !important;
}

.grader-toolbar-button svg {
  width: 7pt;
  height: 7pt;
}

/* old Widgets */

.nbgrader-CreateAssignmentWidget {
  background: var(--jp-layout-color1);
  color: var(--jp-ui-font-color1);
  min-width: 251px;
}

.cell-error {
  background-color: rgb(255 112 112);
}

.nbgrader-NotebookPanelWidget {
  height: 100%;
  display: flex;
  flex-direction: column;
}

.nbgrader-NotebookHeaderWidget {
  padding-top: 12px;
  border-bottom: solid var(--jp-border-color1) var(--jp-border-width);
  box-shadow: var(--jp-toolbar-box-shadow);
}

.nbgrader-NotebookHeaderWidget > * {
  padding: 0 12px;
}

.nbgrader-TotalPointsInput {
  width: 3em;
  height: 24px;
  text-align: right;
  line-height: 10px;
  margin-left: 0.3em;
  margin-right: 0.3em;
  background: var(--jp-input-background);
  font-size: var(--jp-ui-font-size1);
  font-family: var(--jp-content-font-family);
  color: var(--jp-ui-font-color0);
  border: var(--jp-border-width) solid var(--jp-border-color1);
  flex: 1 1 auto;
  padding: 0 8px;
  outline: none;
}

.nbgrader-NotebookWidget {
  overflow: auto;
  height: 100%;
}

.nbgrader-CellWidget {
  margin-top: 12px;
  height: auto;
  border-style: solid;
  border-width: 0 0 var(--jp-border-width) 0;
  border-color: var(--jp-border-color2);
}

.nbgrader-CellWidget:last-child {
  border-width: 0;
}

.nbgrader-CellWidget > div {
  padding: 0 12px 0 6px;
  border-style: solid;
  border-width: 0 0 0 6px;
  border-color: #0000;
}

.nbgrader-CellWidget.nbgrader-mod-active > div {
  border-color: var(--jp-brand-color1);
}

.nbgrader-CellWidget input {
  background: var(--jp-input-background);
  font-size: var(--jp-ui-font-size1);
  font-family: var(--jp-content-font-family);
  color: var(--jp-ui-font-color0);
  border: var(--jp-border-width) solid var(--jp-border-color1);
  flex: 1 1 auto;
  padding: 0 8px;
  outline: none;
}

.nbgrader-CellWidget .jp-select-wrapper {
  display: inline-flex;
  font-size: var(--jp-ui-font-size1);
  margin-bottom: 0;
}

.nbgrader-CellWidget .jp-select-wrapper select.jp-mod-styled {
  font-size: var(--jp-ui-font-size1);
}

.nbgrader-CellWidget .nbgrader-mod-uneditable {
  visibility: hidden;
}

.nbgrader-CellWidget select {
  padding-right: 25px;
}

.nbgrader-CellWidget select:hover {
  cursor: pointer;
}

.nbgrader-NotebookPoints,
.nbgrader-CellHeader,
.nbgrader-CellType,
.nbgrader-CellPoints,
.nbgrader-CellId {
  margin-bottom: 12px;
}

.nbgrader-CellWidget > label {
  font-size: var(--jp-ui-font-size1);
}

.nbgrader-CellHeader {
  display: grid;
  grid-template-columns: auto max-content;
  background-color: #eee;
  padding: 0 0.5em;
}

.nbgrader-mod-highlight .nbgrader-CellHeader {
  background-color: #337ab7;
}

.nbgrader-CellHeader > .jp-InputPrompt {
  border-width: 0;
  padding-left: 0;
  text-align: left;
  margin: auto 0;
}

.nbgrader-CellHeader > * {
  color: black;
}

.nbgrader-mod-highlight .nbgrader-CellHeader > * {
  color: white;
}

.nbgrader-LockButton {
  opacity: 0;
}

.nbgrader-CellHeader > .nbgrader-LockButton {
  margin: auto 0;
  padding: 0.25em;
  cursor: default;
}

.nbgrader-LockButton.nbgrader-mod-locked {
  opacity: 1;
}

.nbgrader-CellPoints input {
  width: 3.5em;
  height: 22px;
  text-align: right;
  line-height: 10px;
}

.nbgrader-CellId input {
  height: 22px;
  text-align: left;
}
