/**
 * Grader Extension API Schemas
 *
 * 
 *
 * NOTE: This class is auto generated by OpenAPI Generator (https://openapi-generator.tech).
 * https://openapi-generator.tech
 * Do not edit the class manually.
 */


export type ManualStatus = 'not_graded' | 'manually_graded' | 'being_edited';

export const ManualStatus = {
    NotGraded: 'not_graded' as ManualStatus,
    ManuallyGraded: 'manually_graded' as ManualStatus,
    BeingEdited: 'being_edited' as ManualStatus
};

