/**
 * Grader Extension API Schemas
 *
 * 
 *
 * NOTE: This class is auto generated by OpenAPI Generator (https://openapi-generator.tech).
 * https://openapi-generator.tech
 * Do not edit the class manually.
 */


export type AutoStatus = 'not_graded' | 'pending' | 'automatically_graded' | 'grading_failed';

export const AutoStatus = {
    NotGraded: 'not_graded' as AutoStatus,
    Pending: 'pending' as AutoStatus,
    AutomaticallyGraded: 'automatically_graded' as AutoStatus,
    GradingFailed: 'grading_failed' as AutoStatus
};

