/*! For license information please see 625.c96c0508fa20b2a5344d.js.LICENSE.txt */
"use strict";(self.webpackChunkgrader_labextension=self.webpackChunkgrader_labextension||[]).push([[625],{12939:(e,o,a)=>{function t(e){return String(parseFloat(e)).length===String(e).length}function n(e){return String(e).match(/[\d.\-+]*\s*(.*)/)[1]||""}function r(e){return parseFloat(e)}function s(e){return(o,a)=>{const t=n(o);if(t===a)return o;let s=r(o);"px"!==t&&("em"===t||"rem"===t)&&(s=r(o)*r(e));let i=s;if("px"!==a)if("em"===a)i=s/r(e);else{if("rem"!==a)return o;i=s/r(e)}return parseFloat(i.toFixed(5))+a}}function i({size:e,grid:o}){const a=e-e%o,t=a+o;return e-a<t-e?a:t}function l({lineHeight:e,pixels:o,htmlFontSize:a}){return o/(e*a)}function c({cssProperty:e,min:o,max:a,unit:t="rem",breakpoints:n=[600,900,1200],transform:r=null}){const s={[e]:`${o}${t}`},i=(a-o)/n[n.length-1];return n.forEach((a=>{let n=o+i*a;null!==r&&(n=r(n)),s[`@media (min-width:${a}px)`]={[e]:`${Math.round(1e4*n)/1e4}${t}`}})),s}a.d(o,{I3:()=>s,VR:()=>i,a9:()=>t,db:()=>r,l_:()=>n,qW:()=>l,yL:()=>c})},28523:(e,o,a)=>{a.d(o,{A:()=>s,E:()=>r});var t=a(38413),n=a(31609);function r(e){return(0,n.Ay)("MuiSkeleton",e)}const s=(0,t.A)("MuiSkeleton",["root","text","rectangular","rounded","circular","pulse","wave","withChildren","fitContent","heightAuto"])},47583:(e,o,a)=>{a.d(o,{A:()=>k});var t=a(93345),n=a(28919),r=a(75659),s=a(12939),i=a(77215),l=a(3556),c=a(32777),u=a(99945),d=a(28523),p=a(74848);const m=i.keyframes`
  0% {
    opacity: 1;
  }

  50% {
    opacity: 0.4;
  }

  100% {
    opacity: 1;
  }
`,b=i.keyframes`
  0% {
    transform: translateX(-100%);
  }

  50% {
    /* +0.5s of delay between each loop */
    transform: translateX(100%);
  }

  100% {
    transform: translateX(100%);
  }
`,f="string"!=typeof m?i.css`
        animation: ${m} 2s ease-in-out 0.5s infinite;
      `:null,h="string"!=typeof b?i.css`
        &::after {
          animation: ${b} 2s linear 0.5s infinite;
        }
      `:null,y=(0,l.Ay)("span",{name:"MuiSkeleton",slot:"Root",overridesResolver:(e,o)=>{const{ownerState:a}=e;return[o.root,o[a.variant],!1!==a.animation&&o[a.animation],a.hasChildren&&o.withChildren,a.hasChildren&&!a.width&&o.fitContent,a.hasChildren&&!a.height&&o.heightAuto]}})((0,c.A)((({theme:e})=>{const o=(0,s.l_)(e.shape.borderRadius)||"px",a=(0,s.db)(e.shape.borderRadius);return{display:"block",backgroundColor:e.vars?e.vars.palette.Skeleton.bg:(0,i.alpha)(e.palette.text.primary,"light"===e.palette.mode?.11:.13),height:"1.2em",variants:[{props:{variant:"text"},style:{marginTop:0,marginBottom:0,height:"auto",transformOrigin:"0 55%",transform:"scale(1, 0.60)",borderRadius:`${a}${o}/${Math.round(a/.6*10)/10}${o}`,"&:empty:before":{content:'"\\00a0"'}}},{props:{variant:"circular"},style:{borderRadius:"50%"}},{props:{variant:"rounded"},style:{borderRadius:(e.vars||e).shape.borderRadius}},{props:({ownerState:e})=>e.hasChildren,style:{"& > *":{visibility:"hidden"}}},{props:({ownerState:e})=>e.hasChildren&&!e.width,style:{maxWidth:"fit-content"}},{props:({ownerState:e})=>e.hasChildren&&!e.height,style:{height:"auto"}},{props:{animation:"pulse"},style:f||{animation:`${m} 2s ease-in-out 0.5s infinite`}},{props:{animation:"wave"},style:{position:"relative",overflow:"hidden",WebkitMaskImage:"-webkit-radial-gradient(white, black)","&::after":{background:`linear-gradient(\n                90deg,\n                transparent,\n                ${(e.vars||e).palette.action.hover},\n                transparent\n              )`,content:'""',position:"absolute",transform:"translateX(-100%)",bottom:0,left:0,right:0,top:0}}},{props:{animation:"wave"},style:h||{"&::after":{animation:`${b} 2s linear 0.5s infinite`}}}]}}))),k=t.forwardRef((function(e,o){const a=(0,u.b)({props:e,name:"MuiSkeleton"}),{animation:t="pulse",className:s,component:i="span",height:l,style:c,variant:m="text",width:b,...f}=a,h={...a,animation:t,component:i,variant:m,hasChildren:Boolean(f.children)},k=(e=>{const{classes:o,variant:a,animation:t,hasChildren:n,width:s,height:i}=e,l={root:["root",a,t,n&&"withChildren",n&&!s&&"fitContent",n&&!i&&"heightAuto"]};return(0,r.A)(l,d.E,o)})(h);return(0,p.jsx)(y,{as:i,ref:o,className:(0,n.A)(k.root,s),ownerState:h,...f,style:{width:b,height:l,...c}})}))},77625:(e,o,a)=>{a.r(o),a.d(o,{ArrowDropDownIcon:()=>Ie.iV,ArrowLeftIcon:()=>Ie.A6,ArrowRightIcon:()=>Ie.fl,CalendarIcon:()=>Ie.CT,ClearIcon:()=>Ie.vu,ClockIcon:()=>Ie.O4,DEFAULT_DESKTOP_MODE_MEDIA_QUERY:()=>$e.jX,DateCalendar:()=>U.x,DateField:()=>I,DatePicker:()=>ze,DatePickerToolbar:()=>Se,DateRangeIcon:()=>Ie.Z5,DateTimeField:()=>V.I,DateTimePicker:()=>yo.K,DateTimePickerTabs:()=>ko.U,DateTimePickerToolbar:()=>To.l,DayCalendarSkeleton:()=>pe,DesktopDatePicker:()=>We,DesktopDateTimePicker:()=>Co.U,DesktopDateTimePickerLayout:()=>vo.U,DesktopTimePicker:()=>po,DigitalClock:()=>l.N,LocalizationProvider:()=>m.$,MobileDatePicker:()=>Ve,MobileDateTimePicker:()=>Ao.E,MobileTimePicker:()=>mo,MonthCalendar:()=>_.T,MuiPickersAdapterContext:()=>m.F,MultiSectionDigitalClock:()=>u.z,PickersActionBar:()=>Mo.L,PickersCalendarHeader:()=>Io.V,PickersDay:()=>b.a,PickersFilledInput:()=>No.W,PickersInput:()=>Fo.a,PickersInputBase:()=>zo.jv,PickersLayout:()=>Ee.CJ,PickersLayoutContentWrapper:()=>Ee.IU,PickersLayoutRoot:()=>Ee.Kr,PickersOutlinedInput:()=>$o.w,PickersShortcuts:()=>Ro.f,PickersTextField:()=>S.X,StaticDatePicker:()=>Ye,StaticDateTimePicker:()=>Oo,StaticTimePicker:()=>ho,TimeClock:()=>t.a,TimeField:()=>N,TimeIcon:()=>Ie.b1,TimePicker:()=>fo,TimePickerToolbar:()=>so,Unstable_PickersSectionList:()=>Eo.fI,Unstable_PickersSectionListRoot:()=>Eo.tl,Unstable_PickersSectionListSection:()=>Eo.er,Unstable_PickersSectionListSectionContent:()=>Eo.vK,Unstable_PickersSectionListSectionSeparator:()=>Eo.jp,YearCalendar:()=>B.i,clockClasses:()=>n.N,clockNumberClasses:()=>r.q,clockPointerClasses:()=>i.S,dateCalendarClasses:()=>z.r,datePickerToolbarClasses:()=>we,dateTimePickerTabsClasses:()=>Po.g,dateTimePickerToolbarClasses:()=>go.d,dayCalendarClasses:()=>Z.f,dayCalendarSkeletonClasses:()=>re,digitalClockClasses:()=>c.X,extractValidationProps:()=>Re.zd,getDateCalendarUtilityClass:()=>z.$,getDayCalendarSkeletonUtilityClass:()=>ne,getDigitalClockUtilityClass:()=>c.L,getMonthCalendarUtilityClass:()=>q.x,getMultiSectionDigitalClockUtilityClass:()=>p.h,getPickersDayUtilityClass:()=>f.m,getPickersFilledInputUtilityClass:()=>Vo.u,getPickersInputBaseUtilityClass:()=>Zo.l,getPickersInputUtilityClass:()=>Wo.e,getPickersOutlinedInputUtilityClass:()=>Uo.G,getPickersSectionListUtilityClass:()=>Xo.b,getPickersTextFieldUtilityClass:()=>Lo.d,getTimeClockUtilityClass:()=>s.Y,getYearCalendarUtilityClass:()=>H.o,monthCalendarClasses:()=>q.b,multiSectionDigitalClockClasses:()=>p.L,multiSectionDigitalClockSectionClasses:()=>d.w,pickersCalendarHeaderClasses:()=>jo.H,pickersDayClasses:()=>f.Q,pickersFadeTransitionGroupClasses:()=>E.r,pickersFilledInputClasses:()=>Vo.C,pickersInputBaseClasses:()=>Zo.R,pickersInputClasses:()=>Wo.a,pickersLayoutClasses:()=>So.k,pickersMonthClasses:()=>Y.m,pickersOutlinedInputClasses:()=>Uo.q,pickersSectionListClasses:()=>Xo.T,pickersSlideTransitionClasses:()=>X.U,pickersTextFieldClasses:()=>Lo.f,pickersYearClasses:()=>K.z,renderDateViewCalendar:()=>Le.W,renderDigitalClockTimeView:()=>lo.$W,renderMultiSectionDigitalClockTimeView:()=>lo.$8,renderTimeViewClock:()=>lo.Zo,timeClockClasses:()=>s.G,timePickerToolbarClasses:()=>Je,unstable_useDateField:()=>O,unstable_useDateTimeField:()=>$.p,unstable_useTimeField:()=>F,useClearableField:()=>x.Z,useParsedFormat:()=>Yo,usePickerLayout:()=>xo.A,usePickersContext:()=>Ho,usePickersTranslations:()=>ve.b,useSplitFieldProps:()=>w.o,useValidation:()=>Ko.$,validateDate:()=>A.Z,validateDateTime:()=>Do.q,validateTime:()=>L.i,yearCalendarClasses:()=>H.E});var t=a(83396),n=a(12454),r=a(72713),s=a(84895),i=a(32127),l=a(44149),c=a(91181),u=a(27899),d=a(15160),p=a(81937),m=a(75737),b=a(63967),f=a(61099),h=a(55285),y=a(40644),k=a(93345),P=a(28183),T=a(92353),g=a(79329),C=a(16566),v=a(24587),A=a(72770),w=a(49940),D=a(9599);const O=e=>{const o=(0,D.LN)(e),{forwardedProps:a,internalProps:t}=(0,w.o)(o,"date");return(0,v.M)({forwardedProps:a,internalProps:t,valueManager:C.W,fieldValueManager:C.o,validator:A.Z,valueType:"date"})};var x=a(57655),S=a(8473),M=a(76879),R=a(74848);const j=["slots","slotProps","InputProps","inputProps"],I=k.forwardRef((function(e,o){const a=(0,T.A)({props:e,name:"MuiDateField"}),{slots:t,slotProps:n,InputProps:r,inputProps:s}=a,i=(0,y.A)(a,j),l=a,c=t?.textField??(e.enableAccessibleFieldDOMStructure?S.X:P.A),u=(0,g.A)({elementType:c,externalSlotProps:n?.textField,externalForwardedProps:i,additionalProps:{ref:o},ownerState:l});u.inputProps=(0,h.A)({},s,u.inputProps),u.InputProps=(0,h.A)({},r,u.InputProps);const d=O(u),p=(0,M.h)(d),m=(0,x.Z)((0,h.A)({},p,{slots:t,slotProps:n}));return(0,R.jsx)(c,(0,h.A)({},m))}));var L=a(14027);const F=e=>{const o=(0,D.EO)(e),{forwardedProps:a,internalProps:t}=(0,w.o)(o,"time");return(0,v.M)({forwardedProps:a,internalProps:t,valueManager:C.W,fieldValueManager:C.o,validator:L.i,valueType:"time"})},W=["slots","slotProps","InputProps","inputProps"],N=k.forwardRef((function(e,o){const a=(0,T.A)({props:e,name:"MuiTimeField"}),{slots:t,slotProps:n,InputProps:r,inputProps:s}=a,i=(0,y.A)(a,W),l=a,c=t?.textField??(e.enableAccessibleFieldDOMStructure?S.X:P.A),u=(0,g.A)({elementType:c,externalSlotProps:n?.textField,externalForwardedProps:i,ownerState:l,additionalProps:{ref:o}});u.inputProps=(0,h.A)({},s,u.inputProps),u.InputProps=(0,h.A)({},r,u.InputProps);const d=F(u),p=(0,M.h)(d),m=(0,x.Z)((0,h.A)({},p,{slots:t,slotProps:n}));return(0,R.jsx)(c,(0,h.A)({},m))}));var V=a(62251),$=a(27176),U=a(27364),z=a(91645),Z=a(18247),E=a(45908),X=a(35790),_=a(4627),q=a(60839),Y=a(51914),B=a(27228),H=a(48443),K=a(66986),Q=a(28919),G=a(47583),J=a(3556),ee=a(75659),oe=a(31886),ae=a(31609),te=a(38413);const ne=e=>(0,ae.Ay)("MuiDayCalendarSkeleton",e),re=(0,te.A)("MuiDayCalendarSkeleton",["root","week","daySkeleton"]),se=["className"],ie=e=>{const{classes:o}=e;return(0,ee.A)({root:["root"],week:["week"],daySkeleton:["daySkeleton"]},ne,o)},le=(0,J.Ay)("div",{name:"MuiDayCalendarSkeleton",slot:"Root",overridesResolver:(e,o)=>o.root})({alignSelf:"start"}),ce=(0,J.Ay)("div",{name:"MuiDayCalendarSkeleton",slot:"Week",overridesResolver:(e,o)=>o.week})({margin:`${oe.y0}px 0`,display:"flex",justifyContent:"center"}),ue=(0,J.Ay)(G.A,{name:"MuiDayCalendarSkeleton",slot:"DaySkeleton",overridesResolver:(e,o)=>o.daySkeleton})({margin:`0 ${oe.y0}px`,variants:[{props:{day:0},style:{visibility:"hidden"}}]}),de=[[0,1,1,1,1,1,1],[1,1,1,1,1,1,1],[1,1,1,1,1,1,1],[1,1,1,1,1,1,1],[1,1,1,1,0,0,0]];function pe(e){const o=(0,T.A)({props:e,name:"MuiDayCalendarSkeleton"}),{className:a}=o,t=(0,y.A)(o,se),n=ie(t);return(0,R.jsx)(le,(0,h.A)({className:(0,Q.A)(n.root,a)},t,{children:de.map(((e,o)=>(0,R.jsx)(ce,{className:n.week,children:e.map(((e,o)=>(0,R.jsx)(ue,{variant:"circular",width:oe.B3,height:oe.B3,className:n.daySkeleton,ownerState:{day:e}},o)))},o)))}))}var me=a(35048),be=a(5556),fe=a.n(be),he=a(56677),ye=a(37021),ke=a(90279),Pe=a(34469),Te=a(23887),ge=a(42819),Ce=a(6498),ve=a(29881);function Ae(e){return(0,ae.Ay)("MuiDatePickerToolbar",e)}const we=(0,te.A)("MuiDatePickerToolbar",["root","title"]),De=["value","isLandscape","onChange","toolbarFormat","toolbarPlaceholder","views","className","onViewChange","view"],Oe=(0,J.Ay)(Ce.Z,{name:"MuiDatePickerToolbar",slot:"Root",overridesResolver:(e,o)=>o.root})({}),xe=(0,J.Ay)(ge.A,{name:"MuiDatePickerToolbar",slot:"Title",overridesResolver:(e,o)=>o.title})({variants:[{props:{isLandscape:!0},style:{margin:"auto 16px auto auto"}}]}),Se=k.forwardRef((function(e,o){const a=(0,T.A)({props:e,name:"MuiDatePickerToolbar"}),{value:t,isLandscape:n,toolbarFormat:r,toolbarPlaceholder:s="––",views:i,className:l}=a,c=(0,y.A)(a,De),u=(0,ke.hc)(),d=(0,ve.b)(),p=(e=>{const{classes:o}=e;return(0,ee.A)({root:["root"],title:["title"]},Ae,o)})(a),m=k.useMemo((()=>{if(!t)return s;const e=(0,Te.Wq)(u,{format:r,views:i},!0);return u.formatByString(t,e)}),[t,r,s,u,i]),b=a;return(0,R.jsx)(Oe,(0,h.A)({ref:o,toolbarTitle:d.datePickerToolbarTitle,isLandscape:n,className:(0,Q.A)(p.root,l)},c,{children:(0,R.jsx)(xe,{variant:"h4",align:n?"left":"center",ownerState:b,className:p.title,children:m})}))}));function Me(e,o){const a=(0,ke.hc)(),t=(0,ke.Yg)(),n=(0,T.A)({props:e,name:o}),r=k.useMemo((()=>null==n.localeText?.toolbarTitle?n.localeText:(0,h.A)({},n.localeText,{datePickerToolbarTitle:n.localeText.toolbarTitle})),[n.localeText]);return(0,h.A)({},n,{localeText:r},(0,Pe.R)({views:n.views,openTo:n.openTo,defaultViews:["year","day"],defaultOpenTo:"day"}),{disableFuture:n.disableFuture??!1,disablePast:n.disablePast??!1,minDate:(0,Te.IP)(a,n.minDate,t.minDate),maxDate:(0,Te.IP)(a,n.maxDate,t.maxDate),slots:(0,h.A)({toolbar:Se},n.slots)})}var Re=a(76108),je=a(80954),Ie=a(40293),Le=a(37755),Fe=a(33230);const We=k.forwardRef((function(e,o){const a=(0,ve.b)(),t=(0,ke.hc)(),n=Me(e,"MuiDesktopDatePicker"),r=(0,h.A)({day:Le.W,month:Le.W,year:Le.W},n.viewRenderers),s=(0,h.A)({},n,{viewRenderers:r,format:(0,Te.Wq)(t,n,!1),yearsPerRow:n.yearsPerRow??4,slots:(0,h.A)({openPickerIcon:Ie.CT,field:I},n.slots),slotProps:(0,h.A)({},n.slotProps,{field:e=>(0,h.A)({},(0,he.A)(n.slotProps?.field,e),(0,Re.zd)(n),{ref:o}),toolbar:(0,h.A)({hidden:!0},n.slotProps?.toolbar)})}),{renderPicker:i}=(0,je.U)({props:s,valueManager:C.W,valueType:"date",getOpenDialogAriaText:(0,Fe.z)({utils:t,formatKey:"fullDate",contextTranslation:a.openDatePickerDialogue,propsTranslation:s.localeText?.openDatePickerDialogue}),validator:A.Z});return i()}));We.propTypes={autoFocus:fe().bool,className:fe().string,closeOnSelect:fe().bool,dayOfWeekFormatter:fe().func,defaultValue:fe().object,disabled:fe().bool,disableFuture:fe().bool,disableHighlightToday:fe().bool,disableOpenPicker:fe().bool,disablePast:fe().bool,displayWeekNumber:fe().bool,enableAccessibleFieldDOMStructure:fe().any,fixedWeekNumber:fe().number,format:fe().string,formatDensity:fe().oneOf(["dense","spacious"]),inputRef:ye.A,label:fe().node,loading:fe().bool,localeText:fe().object,maxDate:fe().object,minDate:fe().object,monthsPerRow:fe().oneOf([3,4]),name:fe().string,onAccept:fe().func,onChange:fe().func,onClose:fe().func,onError:fe().func,onMonthChange:fe().func,onOpen:fe().func,onSelectedSectionsChange:fe().func,onViewChange:fe().func,onYearChange:fe().func,open:fe().bool,openTo:fe().oneOf(["day","month","year"]),orientation:fe().oneOf(["landscape","portrait"]),readOnly:fe().bool,reduceAnimations:fe().bool,referenceDate:fe().object,renderLoading:fe().func,selectedSections:fe().oneOfType([fe().oneOf(["all","day","empty","hours","meridiem","minutes","month","seconds","weekDay","year"]),fe().number]),shouldDisableDate:fe().func,shouldDisableMonth:fe().func,shouldDisableYear:fe().func,showDaysOutsideCurrentMonth:fe().bool,slotProps:fe().object,slots:fe().object,sx:fe().oneOfType([fe().arrayOf(fe().oneOfType([fe().func,fe().object,fe().bool])),fe().func,fe().object]),timezone:fe().string,value:fe().object,view:fe().oneOf(["day","month","year"]),viewRenderers:fe().shape({day:fe().func,month:fe().func,year:fe().func}),views:fe().arrayOf(fe().oneOf(["day","month","year"]).isRequired),yearsOrder:fe().oneOf(["asc","desc"]),yearsPerRow:fe().oneOf([3,4])};var Ne=a(8078);const Ve=k.forwardRef((function(e,o){const a=(0,ve.b)(),t=(0,ke.hc)(),n=Me(e,"MuiMobileDatePicker"),r=(0,h.A)({day:Le.W,month:Le.W,year:Le.W},n.viewRenderers),s=(0,h.A)({},n,{viewRenderers:r,format:(0,Te.Wq)(t,n,!1),slots:(0,h.A)({field:I},n.slots),slotProps:(0,h.A)({},n.slotProps,{field:e=>(0,h.A)({},(0,he.A)(n.slotProps?.field,e),(0,Re.zd)(n),{ref:o}),toolbar:(0,h.A)({hidden:!1},n.slotProps?.toolbar)})}),{renderPicker:i}=(0,Ne.C)({props:s,valueManager:C.W,valueType:"date",getOpenDialogAriaText:(0,Fe.z)({utils:t,formatKey:"fullDate",contextTranslation:a.openDatePickerDialogue,propsTranslation:s.localeText?.openDatePickerDialogue}),validator:A.Z});return i()}));Ve.propTypes={autoFocus:fe().bool,className:fe().string,closeOnSelect:fe().bool,dayOfWeekFormatter:fe().func,defaultValue:fe().object,disabled:fe().bool,disableFuture:fe().bool,disableHighlightToday:fe().bool,disableOpenPicker:fe().bool,disablePast:fe().bool,displayWeekNumber:fe().bool,enableAccessibleFieldDOMStructure:fe().any,fixedWeekNumber:fe().number,format:fe().string,formatDensity:fe().oneOf(["dense","spacious"]),inputRef:ye.A,label:fe().node,loading:fe().bool,localeText:fe().object,maxDate:fe().object,minDate:fe().object,monthsPerRow:fe().oneOf([3,4]),name:fe().string,onAccept:fe().func,onChange:fe().func,onClose:fe().func,onError:fe().func,onMonthChange:fe().func,onOpen:fe().func,onSelectedSectionsChange:fe().func,onViewChange:fe().func,onYearChange:fe().func,open:fe().bool,openTo:fe().oneOf(["day","month","year"]),orientation:fe().oneOf(["landscape","portrait"]),readOnly:fe().bool,reduceAnimations:fe().bool,referenceDate:fe().object,renderLoading:fe().func,selectedSections:fe().oneOfType([fe().oneOf(["all","day","empty","hours","meridiem","minutes","month","seconds","weekDay","year"]),fe().number]),shouldDisableDate:fe().func,shouldDisableMonth:fe().func,shouldDisableYear:fe().func,showDaysOutsideCurrentMonth:fe().bool,slotProps:fe().object,slots:fe().object,sx:fe().oneOfType([fe().arrayOf(fe().oneOfType([fe().func,fe().object,fe().bool])),fe().func,fe().object]),timezone:fe().string,value:fe().object,view:fe().oneOf(["day","month","year"]),viewRenderers:fe().shape({day:fe().func,month:fe().func,year:fe().func}),views:fe().arrayOf(fe().oneOf(["day","month","year"]).isRequired),yearsOrder:fe().oneOf(["asc","desc"]),yearsPerRow:fe().oneOf([3,4])};var $e=a(24034);const Ue=["desktopModeMediaQuery"],ze=k.forwardRef((function(e,o){const a=(0,T.A)({props:e,name:"MuiDatePicker"}),{desktopModeMediaQuery:t=$e.jX}=a,n=(0,y.A)(a,Ue);return(0,me.A)(t,{defaultMatches:!0})?(0,R.jsx)(We,(0,h.A)({ref:o},n)):(0,R.jsx)(Ve,(0,h.A)({ref:o},n))}));var Ze=a(19835),Ee=a(631);const Xe=["props","ref"],_e=(0,J.Ay)(Ee.CJ)((({theme:e})=>({overflow:"hidden",minWidth:oe.eX,backgroundColor:(e.vars||e).palette.background.paper}))),qe=e=>{let{props:o,ref:a}=e,t=(0,y.A)(e,Xe);const{localeText:n,slots:r,slotProps:s,className:i,sx:l,displayStaticWrapperAs:c,autoFocus:u}=o,{layoutProps:d,renderCurrentView:p}=(0,Ze.k)((0,h.A)({},t,{props:o,autoFocusView:u??!1,additionalViewProps:{},wrapperVariant:c})),b=r?.layout??_e;return{renderPicker:()=>(0,R.jsx)(m.$,{localeText:n,children:(0,R.jsx)(b,(0,h.A)({},d,s?.layout,{slots:r,slotProps:s,sx:[...Array.isArray(l)?l:[l],...Array.isArray(s?.layout?.sx)?s.layout.sx:[s?.layout?.sx]],className:(0,Q.A)(i,s?.layout?.className),ref:a,children:p()}))})}},Ye=k.forwardRef((function(e,o){const a=Me(e,"MuiStaticDatePicker"),t=a.displayStaticWrapperAs??"mobile",n=(0,h.A)({day:Le.W,month:Le.W,year:Le.W},a.viewRenderers),r=(0,h.A)({},a,{viewRenderers:n,displayStaticWrapperAs:t,yearsPerRow:a.yearsPerRow??("mobile"===t?3:4),slotProps:(0,h.A)({},a.slotProps,{toolbar:(0,h.A)({hidden:"desktop"===t},a.slotProps?.toolbar)})}),{renderPicker:s}=qe({props:r,valueManager:C.W,valueType:"date",validator:A.Z,ref:o});return s()}));Ye.propTypes={autoFocus:fe().bool,className:fe().string,dayOfWeekFormatter:fe().func,defaultValue:fe().object,disabled:fe().bool,disableFuture:fe().bool,disableHighlightToday:fe().bool,disablePast:fe().bool,displayStaticWrapperAs:fe().oneOf(["desktop","mobile"]),displayWeekNumber:fe().bool,fixedWeekNumber:fe().number,loading:fe().bool,localeText:fe().object,maxDate:fe().object,minDate:fe().object,monthsPerRow:fe().oneOf([3,4]),onAccept:fe().func,onChange:fe().func,onClose:fe().func,onError:fe().func,onMonthChange:fe().func,onViewChange:fe().func,onYearChange:fe().func,openTo:fe().oneOf(["day","month","year"]),orientation:fe().oneOf(["landscape","portrait"]),readOnly:fe().bool,reduceAnimations:fe().bool,referenceDate:fe().object,renderLoading:fe().func,shouldDisableDate:fe().func,shouldDisableMonth:fe().func,shouldDisableYear:fe().func,showDaysOutsideCurrentMonth:fe().bool,slotProps:fe().object,slots:fe().object,sx:fe().oneOfType([fe().arrayOf(fe().oneOfType([fe().func,fe().object,fe().bool])),fe().func,fe().object]),timezone:fe().string,value:fe().object,view:fe().oneOf(["day","month","year"]),viewRenderers:fe().shape({day:fe().func,month:fe().func,year:fe().func}),views:fe().arrayOf(fe().oneOf(["day","month","year"]).isRequired),yearsOrder:fe().oneOf(["asc","desc"]),yearsPerRow:fe().oneOf([3,4])};var Be=a(73788),He=a(28997),Ke=a(8934),Qe=a(50117);function Ge(e){return(0,ae.Ay)("MuiTimePickerToolbar",e)}const Je=(0,te.A)("MuiTimePickerToolbar",["root","separator","hourMinuteLabel","hourMinuteLabelLandscape","hourMinuteLabelReverse","ampmSelection","ampmLandscape","ampmLabel"]),eo=["ampm","ampmInClock","value","isLandscape","onChange","view","onViewChange","views","disabled","readOnly","className"],oo=e=>{const{isLandscape:o,classes:a,isRtl:t}=e,n={root:["root"],separator:["separator"],hourMinuteLabel:["hourMinuteLabel",o&&"hourMinuteLabelLandscape",t&&"hourMinuteLabelReverse"],ampmSelection:["ampmSelection",o&&"ampmLandscape"],ampmLabel:["ampmLabel"]};return(0,ee.A)(n,Ge,a)},ao=(0,J.Ay)(Ce.Z,{name:"MuiTimePickerToolbar",slot:"Root",overridesResolver:(e,o)=>o.root})({}),to=(0,J.Ay)(He.k,{name:"MuiTimePickerToolbar",slot:"Separator",overridesResolver:(e,o)=>o.separator})({outline:0,margin:"0 4px 0 2px",cursor:"default"}),no=(0,J.Ay)("div",{name:"MuiTimePickerToolbar",slot:"HourMinuteLabel",overridesResolver:(e,o)=>[{[`&.${Je.hourMinuteLabelLandscape}`]:o.hourMinuteLabelLandscape,[`&.${Je.hourMinuteLabelReverse}`]:o.hourMinuteLabelReverse},o.hourMinuteLabel]})({display:"flex",justifyContent:"flex-end",alignItems:"flex-end",variants:[{props:{isRtl:!0},style:{flexDirection:"row-reverse"}},{props:{isLandscape:!0},style:{marginTop:"auto"}}]}),ro=(0,J.Ay)("div",{name:"MuiTimePickerToolbar",slot:"AmPmSelection",overridesResolver:(e,o)=>[{[`.${Je.ampmLabel}`]:o.ampmLabel},{[`&.${Je.ampmLandscape}`]:o.ampmLandscape},o.ampmSelection]})({display:"flex",flexDirection:"column",marginRight:"auto",marginLeft:12,[`& .${Je.ampmLabel}`]:{fontSize:17},variants:[{props:{isLandscape:!0},style:{margin:"4px 0 auto",flexDirection:"row",justifyContent:"space-around",flexBasis:"100%"}}]});function so(e){const o=(0,T.A)({props:e,name:"MuiTimePickerToolbar"}),{ampm:a,ampmInClock:t,value:n,isLandscape:r,onChange:s,view:i,onViewChange:l,views:c,disabled:u,readOnly:d,className:p}=o,m=(0,y.A)(o,eo),b=(0,ke.hc)(),f=(0,ve.b)(),k=(0,Be.I)(),P=Boolean(a&&!t&&c.includes("hours")),{meridiemMode:g,handleMeridiemChange:C}=(0,Qe.r7)(n,a,s),v=(0,h.A)({},o,{isRtl:k}),A=oo(v),w=(0,R.jsx)(to,{tabIndex:-1,value:":",variant:"h3",selected:!1,className:A.separator});return(0,R.jsxs)(ao,(0,h.A)({landscapeDirection:"row",toolbarTitle:f.timePickerToolbarTitle,isLandscape:r,ownerState:v,className:(0,Q.A)(A.root,p)},m,{children:[(0,R.jsxs)(no,{className:A.hourMinuteLabel,ownerState:v,children:[(0,$e.Xk)(c,"hours")&&(0,R.jsx)(Ke.R,{tabIndex:-1,variant:"h3",onClick:()=>l("hours"),selected:"hours"===i,value:n?(D=n,a?b.format(D,"hours12h"):b.format(D,"hours24h")):"--"}),(0,$e.Xk)(c,["hours","minutes"])&&w,(0,$e.Xk)(c,"minutes")&&(0,R.jsx)(Ke.R,{tabIndex:-1,variant:"h3",onClick:()=>l("minutes"),selected:"minutes"===i,value:n?b.format(n,"minutes"):"--"}),(0,$e.Xk)(c,["minutes","seconds"])&&w,(0,$e.Xk)(c,"seconds")&&(0,R.jsx)(Ke.R,{variant:"h3",onClick:()=>l("seconds"),selected:"seconds"===i,value:n?b.format(n,"seconds"):"--"})]}),P&&(0,R.jsxs)(ro,{className:A.ampmSelection,ownerState:v,children:[(0,R.jsx)(Ke.R,{disableRipple:!0,variant:"subtitle2",selected:"am"===g,typographyClassName:A.ampmLabel,value:(0,Te._S)(b,"am"),onClick:d?void 0:()=>C("am"),disabled:u}),(0,R.jsx)(Ke.R,{disableRipple:!0,variant:"subtitle2",selected:"pm"===g,typographyClassName:A.ampmLabel,value:(0,Te._S)(b,"pm"),onClick:d?void 0:()=>C("pm"),disabled:u})]})]}));var D}function io(e,o){const a=(0,ke.hc)(),t=(0,T.A)({props:e,name:o}),n=t.ampm??a.is12HourCycleInCurrentLocale(),r=k.useMemo((()=>null==t.localeText?.toolbarTitle?t.localeText:(0,h.A)({},t.localeText,{timePickerToolbarTitle:t.localeText.toolbarTitle})),[t.localeText]);return(0,h.A)({},t,{ampm:n,localeText:r},(0,Pe.R)({views:t.views,openTo:t.openTo,defaultViews:["hours","minutes"],defaultOpenTo:"hours"}),{disableFuture:t.disableFuture??!1,disablePast:t.disablePast??!1,slots:(0,h.A)({toolbar:so},t.slots),slotProps:(0,h.A)({},t.slotProps,{toolbar:(0,h.A)({ampm:n,ampmInClock:t.ampmInClock},t.slotProps?.toolbar)})})}var lo=a(50771),co=a(95878),uo=a(81589);const po=k.forwardRef((function(e,o){const a=(0,ve.b)(),t=(0,ke.hc)(),n=io(e,"MuiDesktopTimePicker"),{shouldRenderTimeInASingleColumn:r,views:s,timeSteps:i}=(0,uo.H)(n),l=r?lo.$W:lo.$8,c=(0,h.A)({hours:l,minutes:l,seconds:l,meridiem:l},n.viewRenderers),u=n.ampmInClock??!0,d=r?[]:["accept"],p=c.hours?.name===lo.$8.name?s:s.filter((e=>"meridiem"!==e)),m=(0,h.A)({},n,{ampmInClock:u,timeSteps:i,viewRenderers:c,format:(0,co.b$)(t,n),views:r?["hours"]:p,slots:(0,h.A)({field:N,openPickerIcon:Ie.O4},n.slots),slotProps:(0,h.A)({},n.slotProps,{field:e=>(0,h.A)({},(0,he.A)(n.slotProps?.field,e),(0,Re.zd)(n),{ref:o}),toolbar:(0,h.A)({hidden:!0,ampmInClock:u},n.slotProps?.toolbar),actionBar:(0,h.A)({actions:d},n.slotProps?.actionBar)})}),{renderPicker:b}=(0,je.U)({props:m,valueManager:C.W,valueType:"time",getOpenDialogAriaText:(0,Fe.z)({utils:t,formatKey:"fullTime",contextTranslation:a.openTimePickerDialogue,propsTranslation:m.localeText?.openTimePickerDialogue}),validator:L.i});return b()}));po.propTypes={ampm:fe().bool,ampmInClock:fe().bool,autoFocus:fe().bool,className:fe().string,closeOnSelect:fe().bool,defaultValue:fe().object,disabled:fe().bool,disableFuture:fe().bool,disableIgnoringDatePartForTimeValidation:fe().bool,disableOpenPicker:fe().bool,disablePast:fe().bool,enableAccessibleFieldDOMStructure:fe().any,format:fe().string,formatDensity:fe().oneOf(["dense","spacious"]),inputRef:ye.A,label:fe().node,localeText:fe().object,maxTime:fe().object,minTime:fe().object,minutesStep:fe().number,name:fe().string,onAccept:fe().func,onChange:fe().func,onClose:fe().func,onError:fe().func,onOpen:fe().func,onSelectedSectionsChange:fe().func,onViewChange:fe().func,open:fe().bool,openTo:fe().oneOf(["hours","meridiem","minutes","seconds"]),orientation:fe().oneOf(["landscape","portrait"]),readOnly:fe().bool,reduceAnimations:fe().bool,referenceDate:fe().object,selectedSections:fe().oneOfType([fe().oneOf(["all","day","empty","hours","meridiem","minutes","month","seconds","weekDay","year"]),fe().number]),shouldDisableTime:fe().func,skipDisabled:fe().bool,slotProps:fe().object,slots:fe().object,sx:fe().oneOfType([fe().arrayOf(fe().oneOfType([fe().func,fe().object,fe().bool])),fe().func,fe().object]),thresholdToRenderTimeInASingleColumn:fe().number,timeSteps:fe().shape({hours:fe().number,minutes:fe().number,seconds:fe().number}),timezone:fe().string,value:fe().object,view:fe().oneOf(["hours","meridiem","minutes","seconds"]),viewRenderers:fe().shape({hours:fe().func,meridiem:fe().func,minutes:fe().func,seconds:fe().func}),views:fe().arrayOf(fe().oneOf(["hours","minutes","seconds"]).isRequired)};const mo=k.forwardRef((function(e,o){const a=(0,ve.b)(),t=(0,ke.hc)(),n=io(e,"MuiMobileTimePicker"),r=(0,h.A)({hours:lo.Zo,minutes:lo.Zo,seconds:lo.Zo},n.viewRenderers),s=n.ampmInClock??!1,i=(0,h.A)({},n,{ampmInClock:s,viewRenderers:r,format:(0,co.b$)(t,n),slots:(0,h.A)({field:N},n.slots),slotProps:(0,h.A)({},n.slotProps,{field:e=>(0,h.A)({},(0,he.A)(n.slotProps?.field,e),(0,Re.zd)(n),{ref:o}),toolbar:(0,h.A)({hidden:!1,ampmInClock:s},n.slotProps?.toolbar)})}),{renderPicker:l}=(0,Ne.C)({props:i,valueManager:C.W,valueType:"time",getOpenDialogAriaText:(0,Fe.z)({utils:t,formatKey:"fullTime",contextTranslation:a.openTimePickerDialogue,propsTranslation:i.localeText?.openTimePickerDialogue}),validator:L.i});return l()}));mo.propTypes={ampm:fe().bool,ampmInClock:fe().bool,autoFocus:fe().bool,className:fe().string,closeOnSelect:fe().bool,defaultValue:fe().object,disabled:fe().bool,disableFuture:fe().bool,disableIgnoringDatePartForTimeValidation:fe().bool,disableOpenPicker:fe().bool,disablePast:fe().bool,enableAccessibleFieldDOMStructure:fe().any,format:fe().string,formatDensity:fe().oneOf(["dense","spacious"]),inputRef:ye.A,label:fe().node,localeText:fe().object,maxTime:fe().object,minTime:fe().object,minutesStep:fe().number,name:fe().string,onAccept:fe().func,onChange:fe().func,onClose:fe().func,onError:fe().func,onOpen:fe().func,onSelectedSectionsChange:fe().func,onViewChange:fe().func,open:fe().bool,openTo:fe().oneOf(["hours","minutes","seconds"]),orientation:fe().oneOf(["landscape","portrait"]),readOnly:fe().bool,reduceAnimations:fe().bool,referenceDate:fe().object,selectedSections:fe().oneOfType([fe().oneOf(["all","day","empty","hours","meridiem","minutes","month","seconds","weekDay","year"]),fe().number]),shouldDisableTime:fe().func,slotProps:fe().object,slots:fe().object,sx:fe().oneOfType([fe().arrayOf(fe().oneOfType([fe().func,fe().object,fe().bool])),fe().func,fe().object]),timezone:fe().string,value:fe().object,view:fe().oneOf(["hours","minutes","seconds"]),viewRenderers:fe().shape({hours:fe().func,minutes:fe().func,seconds:fe().func}),views:fe().arrayOf(fe().oneOf(["hours","minutes","seconds"]).isRequired)};const bo=["desktopModeMediaQuery"],fo=k.forwardRef((function(e,o){const a=(0,T.A)({props:e,name:"MuiTimePicker"}),{desktopModeMediaQuery:t=$e.jX}=a,n=(0,y.A)(a,bo);return(0,me.A)(t,{defaultMatches:!0})?(0,R.jsx)(po,(0,h.A)({ref:o},n)):(0,R.jsx)(mo,(0,h.A)({ref:o},n))})),ho=k.forwardRef((function(e,o){const a=io(e,"MuiStaticTimePicker"),t=a.displayStaticWrapperAs??"mobile",n=a.ampmInClock??"desktop"===t,r=(0,h.A)({hours:lo.Zo,minutes:lo.Zo,seconds:lo.Zo},a.viewRenderers),s=(0,h.A)({},a,{viewRenderers:r,displayStaticWrapperAs:t,ampmInClock:n,slotProps:(0,h.A)({},a.slotProps,{toolbar:(0,h.A)({hidden:"desktop"===t,ampmInClock:n},a.slotProps?.toolbar)})}),{renderPicker:i}=qe({props:s,valueManager:C.W,valueType:"time",validator:L.i,ref:o});return i()}));ho.propTypes={ampm:fe().bool,ampmInClock:fe().bool,autoFocus:fe().bool,className:fe().string,defaultValue:fe().object,disabled:fe().bool,disableFuture:fe().bool,disableIgnoringDatePartForTimeValidation:fe().bool,disablePast:fe().bool,displayStaticWrapperAs:fe().oneOf(["desktop","mobile"]),localeText:fe().object,maxTime:fe().object,minTime:fe().object,minutesStep:fe().number,onAccept:fe().func,onChange:fe().func,onClose:fe().func,onError:fe().func,onViewChange:fe().func,openTo:fe().oneOf(["hours","minutes","seconds"]),orientation:fe().oneOf(["landscape","portrait"]),readOnly:fe().bool,reduceAnimations:fe().bool,referenceDate:fe().object,shouldDisableTime:fe().func,slotProps:fe().object,slots:fe().object,sx:fe().oneOfType([fe().arrayOf(fe().oneOfType([fe().func,fe().object,fe().bool])),fe().func,fe().object]),timezone:fe().string,value:fe().object,view:fe().oneOf(["hours","minutes","seconds"]),viewRenderers:fe().shape({hours:fe().func,minutes:fe().func,seconds:fe().func}),views:fe().arrayOf(fe().oneOf(["hours","minutes","seconds"]).isRequired)};var yo=a(73643),ko=a(70407),Po=a(58099),To=a(6476),go=a(26438),Co=a(82723),vo=a(67893),Ao=a(47963),wo=a(70333),Do=a(98823);const Oo=k.forwardRef((function(e,o){const a=(0,wo.u)(e,"MuiStaticDateTimePicker"),t=a.displayStaticWrapperAs??"mobile",n=a.ampmInClock??"desktop"===t,r=(0,h.A)({day:Le.W,month:Le.W,year:Le.W,hours:lo.Zo,minutes:lo.Zo,seconds:lo.Zo},a.viewRenderers),s=(0,h.A)({},a,{viewRenderers:r,displayStaticWrapperAs:t,ampmInClock:n,yearsPerRow:a.yearsPerRow??("mobile"===t?3:4),slotProps:(0,h.A)({},a.slotProps,{tabs:(0,h.A)({hidden:"desktop"===t},a.slotProps?.tabs),toolbar:(0,h.A)({hidden:"desktop"===t,ampmInClock:n},a.slotProps?.toolbar)})}),{renderPicker:i}=qe({props:s,valueManager:C.W,valueType:"date-time",validator:Do.q,ref:o});return i()}));Oo.propTypes={ampm:fe().bool,ampmInClock:fe().bool,autoFocus:fe().bool,className:fe().string,dayOfWeekFormatter:fe().func,defaultValue:fe().object,disabled:fe().bool,disableFuture:fe().bool,disableHighlightToday:fe().bool,disableIgnoringDatePartForTimeValidation:fe().bool,disablePast:fe().bool,displayStaticWrapperAs:fe().oneOf(["desktop","mobile"]),displayWeekNumber:fe().bool,fixedWeekNumber:fe().number,loading:fe().bool,localeText:fe().object,maxDate:fe().object,maxDateTime:fe().object,maxTime:fe().object,minDate:fe().object,minDateTime:fe().object,minTime:fe().object,minutesStep:fe().number,monthsPerRow:fe().oneOf([3,4]),onAccept:fe().func,onChange:fe().func,onClose:fe().func,onError:fe().func,onMonthChange:fe().func,onViewChange:fe().func,onYearChange:fe().func,openTo:fe().oneOf(["day","hours","minutes","month","seconds","year"]),orientation:fe().oneOf(["landscape","portrait"]),readOnly:fe().bool,reduceAnimations:fe().bool,referenceDate:fe().object,renderLoading:fe().func,shouldDisableDate:fe().func,shouldDisableMonth:fe().func,shouldDisableTime:fe().func,shouldDisableYear:fe().func,showDaysOutsideCurrentMonth:fe().bool,slotProps:fe().object,slots:fe().object,sx:fe().oneOfType([fe().arrayOf(fe().oneOfType([fe().func,fe().object,fe().bool])),fe().func,fe().object]),timezone:fe().string,value:fe().object,view:fe().oneOf(["day","hours","minutes","month","seconds","year"]),viewRenderers:fe().shape({day:fe().func,hours:fe().func,minutes:fe().func,month:fe().func,seconds:fe().func,year:fe().func}),views:fe().arrayOf(fe().oneOf(["day","hours","minutes","month","seconds","year"]).isRequired),yearsOrder:fe().oneOf(["asc","desc"]),yearsPerRow:fe().oneOf([3,4])};var xo=a(73721),So=a(89827),Mo=a(29885),Ro=a(1909),jo=a(80199),Io=a(43059),Lo=a(69193),Fo=a(15402),Wo=a(84424),No=a(49622),Vo=a(69708),$o=a(85066),Uo=a(98336),zo=a(15672),Zo=a(9642),Eo=a(53013),Xo=a(39117),_o=a(40185),qo=a(91494);const Yo=e=>{const{format:o,formatDensity:a="dense",shouldRespectLeadingZeros:t=!1}=e,n=(0,ke.hc)(),r=(0,Be.I)(),s=(0,ve.b)(),i=k.useMemo((()=>(0,qo.wk)(n)),[n]);return k.useMemo((()=>(0,_o.S)({utils:n,format:o,formatDensity:a,isRtl:r,shouldRespectLeadingZeros:t,localeText:s,localizedDigits:i,date:null,enableAccessibleFieldDOMStructure:!1}).map((e=>`${e.startSeparator}${e.placeholder}${e.endSeparator}`)).join("")),[n,r,s,i,o,a,t])};var Bo=a(99892);const Ho=()=>{const e=k.useContext(Bo.L);if(null==e)throw new Error(["MUI X: The `usePickersContext` can only be called in fields that are used as a slot of a picker component"].join("\n"));return e};var Ko=a(20362)}}]);