/**
 * Grader Extension API Schemas
 *
 * 
 *
 * NOTE: This class is auto generated by OpenAPI Generator (https://openapi-generator.tech).
 * https://openapi-generator.tech
 * Do not edit the class manually.
 */


export type FeedbackStatus = 'not_generated' | 'generating' | 'generated' | 'generation_failed' | 'feedback_outdated';

export const FeedbackStatus = {
    NotGenerated: 'not_generated' as FeedbackStatus,
    Generating: 'generating' as FeedbackStatus,
    Generated: 'generated' as FeedbackStatus,
    GenerationFailed: 'generation_failed' as FeedbackStatus,
    FeedbackOutdated: 'feedback_outdated' as FeedbackStatus
};

