"use strict";(self.webpackChunkgrader_labextension=self.webpackChunkgrader_labextension||[]).push([[169,550],{15327:(e,t,n)=>{n.d(t,{A:()=>m,T:()=>d});var r=n(93345),o=n(21529),s=n(37340),a=n(25463);function u(e,t,n,s,a){const[u,c]=r.useState((()=>a&&n?n(e).matches:s?s(e).matches:t));return(0,o.A)((()=>{if(!n)return;const t=n(e),r=()=>{c(t.matches)};return r(),t.addEventListener("change",r),()=>{t.removeEventListener("change",r)}}),[e,n]),u}const c={...r}.useSyncExternalStore;function i(e,t,n,o,s){const a=r.useCallback((()=>t),[t]),u=r.useMemo((()=>{if(s&&n)return()=>n(e).matches;if(null!==o){const{matches:t}=o(e);return()=>t}return a}),[a,e,o,s,n]),[i,d]=r.useMemo((()=>{if(null===n)return[a,()=>()=>{}];const t=n(e);return[()=>t.matches,e=>(t.addEventListener("change",e),()=>{t.removeEventListener("change",e)})]}),[a,n,e]);return c(d,i,u)}function d(e={}){const{themeId:t}=e;return function(e,n={}){let r=(0,a.A)();r&&t&&(r=r[t]||r);const o="undefined"!=typeof window&&void 0!==window.matchMedia,{defaultMatches:d=!1,matchMedia:m=(o?window.matchMedia:null),ssrMatchMedia:f=null,noSsr:l=!1}=(0,s.A)({name:"MuiUseMediaQuery",props:n,theme:r});let h="function"==typeof e?e(r):e;return h=h.replace(/^@media( ?)/m,""),(void 0!==c?i:u)(h,d,m,f,l)}}const m=d()},21529:(e,t,n)=>{n.d(t,{A:()=>o});var r=n(93345);const o="undefined"!=typeof window?r.useLayoutEffect:r.useEffect},25463:(e,t,n)=>{n.d(t,{A:()=>s});var r=n(93345),o=n(83157);const s=function(e=null){const t=r.useContext(o.ThemeContext);return t&&(n=t,0!==Object.keys(n).length)?t:e;var n}},28473:(e,t,n)=>{n.d(t,{A:()=>o});var r=n(93345);function o(e,t){return r.isValidElement(e)&&-1!==t.indexOf(e.type.muiName??e.type?._payload?.value?.muiName)}},37340:(e,t,n)=>{n.d(t,{A:()=>o});var r=n(47379);function o(e){const{theme:t,name:n,props:o}=e;return t&&t.components&&t.components[n]&&t.components[n].defaultProps?(0,r.A)(t.components[n].defaultProps,o):o}},39599:(e,t,n)=>{n.d(t,{A:()=>a});var r=n(11317),o=n(37709);const s=e=>{const t={systemProps:{},otherProps:{}},n=e?.theme?.unstable_sxConfig??o.A;return Object.keys(e).forEach((r=>{n[r]?t.systemProps[r]=e[r]:t.otherProps[r]=e[r]})),t};function a(e){const{sx:t,...n}=e,{systemProps:o,otherProps:a}=s(n);let u;return u=Array.isArray(t)?[o,...t]:"function"==typeof t?(...e)=>{const n=t(...e);return(0,r.Q)(n)?{...o,...n}:o}:{...o,...t},{...a,sx:u}}},40973:(e,t,n)=>{n.d(t,{A:()=>a});var r=n(49765),o=n(25463);const s=(0,r.A)(),a=function(e=s){return(0,o.A)(e)}},44467:(e,t,n)=>{n.d(t,{A:()=>s});var r=n(37340),o=n(40973);function s({props:e,name:t,defaultTheme:n,themeId:s}){let a=(0,o.A)(n);return s&&(a=a[s]||a),(0,r.A)({theme:a,name:t,props:e})}},73788:(e,t,n)=>{n.d(t,{A:()=>u,I:()=>a});var r=n(93345),o=n(74848);const s=r.createContext(),a=()=>r.useContext(s)??!1,u=function({value:e,...t}){return(0,o.jsx)(s.Provider,{value:e??!0,...t})}}}]);