/**
 * Grader Extension API Schemas
 *
 * 
 *
 * NOTE: This class is auto generated by OpenAPI Generator (https://openapi-generator.tech).
 * https://openapi-generator.tech
 * Do not edit the class manually.
 */


export interface RemoteFileStatus { 
    status: RemoteFileStatus.StatusEnum;
}
export namespace RemoteFileStatus {
    export const StatusEnum = {
        UpToDate: 'UP_TO_DATE',
        Divergent: 'DIVERGENT',
        PullNeeded: 'PULL_NEEDED',
        PushNeeded: 'PUSH_NEEDED',
        NoRemoteRepo: 'NO_REMOTE_REPO'
    } as const;
    export type StatusEnum = typeof StatusEnum[keyof typeof StatusEnum];
}


