/**
 * Grader Extension API Schemas
 *
 * 
 *
 * NOTE: This class is auto generated by OpenAPI Generator (https://openapi-generator.tech).
 * https://openapi-generator.tech
 * Do not edit the class manually.
 */
import { User } from './user';
import { Submission } from './submission';


export interface UserSubmissionsInner { 
    user?: User;
    submissions?: Array<Submission>;
}

